<?php

namespace CRPlugins\Oca\Orders;

use CRPlugins\Oca\Helper\Helper;
use CRPlugins\Oca\ShippingLabels\Pdf10x15LabelsProcessor;
use CRPlugins\Oca\ShippingLabels\PdfA4LabelsProcessor;
use CRPlugins\Oca\ShippingLabels\ZplLabelsProcessor;
use CRPlugins\Oca\ShippingMethod\OrderProcessor;
use DateTime;
use Exception;

defined( 'ABSPATH' ) || exit;

class AjaxCalls {

		/**
	 * @var PdfA4LabelsProcessor
	 */
	private $pdf_a4_processor;

	/**
	 * @var Pdf10x15LabelsProcessor
	 */
	private $pdf_10x15_processor;

	/**
	 * @var ZplLabelsProcessor
	 */
	private $zpl_processor;

	/**
	 * @var OrderProcessor
	 */
	private $order_processor;

	public function __construct() {
		add_action( 'wp_ajax_oca_get_shipping_label_pdf', array( $this, 'get_shipping_label_pdf' ) );
		add_action( 'wp_ajax_oca_get_shipping_label_zpl', array( $this, 'get_shipping_label_zpl' ) );
		add_action( 'wp_ajax_oca_get_multiple_shipping_labels_pdf', array( $this, 'get_multiple_shipping_labels_pdf' ) );
		add_action( 'wp_ajax_oca_download_multiple_shipping_labels_pdf', array( $this, 'download_multiple_shipping_labels_pdf' ) );
		add_action( 'wp_ajax_oca_download_multiple_shipping_labels_zpl', array( $this, 'download_multiple_shipping_labels_zpl' ) );
		add_action( 'wp_ajax_oca_get_order_tracking', array( $this, 'get_order_tracking' ) );
		add_action( 'wp_ajax_oca_process_order', array( $this, 'process_order' ) );
		add_action( 'wp_ajax_oca_cancel_order', array( $this, 'cancel_order' ) );
		add_action( 'wp_ajax_oca_update_order_packages_number', array( $this, 'update_order_packages_number' ) );
		add_action( 'wp_ajax_oca_requote_order', array( $this, 'requote_order' ) );

		$this->pdf_a4_processor    = new PdfA4LabelsProcessor();
		$this->pdf_10x15_processor = new Pdf10x15LabelsProcessor();
		$this->zpl_processor       = new ZplLabelsProcessor();
		$this->order_processor     = new OrderProcessor();
	}

	public function requote_order() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_id'] ) ) {
			wp_send_json_error();
		}

		$order_id = filter_var( wp_unslash( $_POST['order_id'] ), FILTER_SANITIZE_NUMBER_INT );
		$order    = wc_get_order( $order_id );
		if ( ! $order ) {
			wp_send_json_error();
		}

		$quoter = new Quoter();
		$price  = $quoter->quote_order( $order );
		if ( empty( $price ) ) {
			wp_send_json_error();
		}

		$price = array_shift( $price );

		wp_send_json_success(
			array(
				'new_quote' => (float) $price['Total'],
				'old_quote' => (float) $order->get_shipping_total() + (float) $order->get_shipping_tax(),
			)
		);
	}

	public function get_shipping_label_pdf() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_id'] ) ) {
			wp_send_json_error();
		}

		$order_id = filter_var( wp_unslash( $_POST['order_id'] ), FILTER_SANITIZE_NUMBER_INT );
		$order    = wc_get_order( $order_id );
		if ( ! $order ) {
			wp_send_json_error();
		}

		$size = 'A4';
		if ( isset( $_POST['size'] ) ) {
			$size = sanitize_text_field( wp_unslash( $_POST['size'] ) );
		}

		if ( 'A4' === $size ) {
			$this->pdf_a4_processor->create_label( $order );
			$shipping_label = $this->pdf_a4_processor->get_label_content( $order );
		} else {
			$this->pdf_10x15_processor->create_label( $order );
			$shipping_label = $this->pdf_10x15_processor->get_label_content( $order );
		}

		if ( empty( $shipping_label ) ) {
			wp_send_json_error();
		}

		wp_send_json_success( base64_encode( $shipping_label ) );
	}

	public function get_shipping_label_zpl() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_id'] ) ) {
			wp_send_json_error();
		}

		$order_id = filter_var( wp_unslash( $_POST['order_id'] ), FILTER_SANITIZE_NUMBER_INT );
		$order    = wc_get_order( $order_id );
		if ( ! $order ) {
			wp_send_json_error();
		}

		$this->zpl_processor->create_label( $order );
		$shipping_label = $this->zpl_processor->get_label_content( $order );

		if ( empty( $shipping_label ) ) {
			wp_send_json_error();
		}

		wp_send_json_success( $shipping_label );
	}

	public function download_multiple_shipping_labels_pdf() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_ids'] ) ) {
			wp_send_json_error();
		}

		$order_ids = sanitize_text_field( wp_unslash( $_POST['order_ids'] ) );
		$order_ids = explode( ',', $order_ids );
		$pdfs      = array();

		$size = 'A4';
		if ( isset( $_POST['size'] ) ) {
			$size = sanitize_text_field( wp_unslash( $_POST['size'] ) );
		}

		foreach ( $order_ids as $order_id ) {
			$order = wc_get_order( $order_id );
			if ( ! $order ) {
				continue;
			}

			if ( ! Helper::order_is_sent_with_oca( $order ) ) {
				continue;
			}

			if ( 'A4' === $size ) {
				$this->pdf_a4_processor->create_label( $order );
				$pdfs[] = $this->pdf_a4_processor->get_label_path( $order );
			} else {
				$this->pdf_10x15_processor->create_label( $order );
				$pdfs[] = $this->pdf_10x15_processor->get_label_path( $order );
			}
		}

		if ( empty( $pdfs ) ) {
			wp_send_json_error();
		}

		try {
			$zip_name = sprintf( __( 'pdf-labels-%s', 'wc-oca' ), ( new DateTime() )->format( 'd-m-Y' ) );
			Helper::create_zip( $zip_name, $pdfs );
		} catch ( Exception $e ) {
			Helper::log_error( 'Could not create compressed PDF: ' . $e->getMessage() );
			wp_send_json_error();
		}

		wp_send_json_success( Helper::get_zip_url( $zip_name ) );
	}

	public function download_multiple_shipping_labels_zpl() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_ids'] ) ) {
			wp_send_json_error();
		}

		$order_ids = sanitize_text_field( wp_unslash( $_POST['order_ids'] ) );
		$order_ids = explode( ',', $order_ids );
		$zpls      = array();

		foreach ( $order_ids as $order_id ) {
			$order = wc_get_order( $order_id );
			if ( ! $order ) {
				continue;
			}

			if ( ! Helper::order_is_sent_with_oca( $order ) ) {
				continue;
			}

			$this->zpl_processor->create_label( $order );
			$zpls[] = $this->zpl_processor->get_label_path( $order );
		}

		if ( empty( $zpls ) ) {
			wp_send_json_error();
		}

		try {
			$zip_name = sprintf( __( 'zpl-labels-%s', 'wc-oca' ), ( new DateTime() )->format( 'd-m-Y' ) );
			Helper::create_zip( $zip_name, $zpls );
		} catch ( Exception $e ) {
			Helper::log_error( 'Could not create compressed PDF: ' . $e->getMessage() );
			wp_send_json_error();
		}

		wp_send_json_success( Helper::get_zip_url( $zip_name ) );
	}

	public function get_multiple_shipping_labels_pdf() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_ids'] ) ) {
			wp_send_json_error();
		}

		$order_ids = sanitize_text_field( wp_unslash( $_POST['order_ids'] ) );
		$order_ids = explode( ',', $order_ids );
		$pdfs      = array();

		$size = 'A4';
		if ( isset( $_POST['size'] ) ) {
			$size = sanitize_text_field( wp_unslash( $_POST['size'] ) );
		}

		foreach ( $order_ids as $order_id ) {
			$order = wc_get_order( $order_id );
			if ( ! $order ) {
				continue;
			}

			if ( ! Helper::order_is_sent_with_oca( $order ) ) {
				continue;
			}

			if ( 'A4' === $size ) {
				$this->pdf_a4_processor->create_label( $order );
				$filename = basename( $this->pdf_a4_processor->get_label_path( $order ) );
			} else {
				$this->pdf_10x15_processor->create_label( $order );
				$filename = basename( $this->pdf_10x15_processor->get_label_path( $order ) );
			}

			if ( file_exists( $filename ) ) {
				$pdfs[] = $filename;
			}
		}
		if ( empty( $pdfs ) ) {
			wp_send_json_error();
		}

		try {
			$content = Helper::create_merged_pdf( $pdfs );
		} catch ( Exception $e ) {
			Helper::log_error( 'Could not create merged PDF: ' . $e->getMessage() );
			wp_send_json_error();
		}

		wp_send_json_success( base64_encode( $content ) );
	}

	public function get_order_tracking() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['trackingNumber'] ) ) {
			wp_send_json_error();
		}

		$tracking_number = filter_var( wp_unslash( $_POST['trackingNumber'] ), FILTER_SANITIZE_NUMBER_INT );
		$output          = do_shortcode( '[oca_order_tracking with_form="no" tracking_number="' . $tracking_number . '"]' );
		wp_send_json_success( $output );
	}

	public function process_order() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_id'] ) || ! isset( $_POST['force_process'] ) ) {
			wp_send_json_error();
		}

		$order_id = filter_var( wp_unslash( $_POST['order_id'] ), FILTER_SANITIZE_NUMBER_INT );
		$order    = wc_get_order( $order_id );
		if ( ! $order ) {
			wp_send_json_error();
		}
		$opts = array( 'force_process' => wp_unslash( $_POST['force_process'] ) );

		$this->order_processor->process_order( $order, $opts );
		wp_send_json_success();
	}

	public function cancel_order() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_id'] ) ) {
			wp_send_json_error();
		}

		$order_id = filter_var( wp_unslash( $_POST['order_id'] ), FILTER_SANITIZE_NUMBER_INT );
		$order    = wc_get_order( $order_id );
		if ( ! $order ) {
			wp_send_json_error();
		}

		$this->order_processor->cancel_order( $order );
		wp_send_json_success();
	}

	public function update_order_packages_number() {
		if ( ! wp_verify_nonce( $_POST['nonce'], 'wc-oca' ) || empty( $_POST['order_id'] ) || empty( $_POST['packages'] ) ) {
			wp_send_json_error();
		}

		$packages = filter_var( wp_unslash( $_POST['packages'] ), FILTER_SANITIZE_NUMBER_INT );
		$order_id = filter_var( wp_unslash( $_POST['order_id'] ), FILTER_SANITIZE_NUMBER_INT );
		$order    = wc_get_order( $order_id );
		if ( ! $order ) {
			wp_send_json_error();
		}

		$shipping_method = Helper::get_order_shipping_method( $order );
		$old_packages    = filter_var( $shipping_method->get_meta( 'oca_packages' ), FILTER_SANITIZE_NUMBER_INT );

		$shipping_method->update_meta_data( 'oca_packages', $packages );
		$shipping_method->save();

		$order->add_order_note( sprintf( __( 'OCA - Number of packages to be sent has been modified from %1$d to %2$d', 'wc-oca' ), $old_packages, $packages ) );
		Helper::log_info( sprintf( __( 'Order %1$d - Number of packages to be sent has been modified from %2$d to %3$d', 'wc-oca' ), $order_id, $old_packages, $packages ) );

		wp_send_json_success();
	}
}
