<?php

namespace CRPlugins\Oca\ShippingLabels;

use CRPlugins\Oca\Helper\Helper;
use CRPlugins\Oca\Sdk\OcaSdk;
use WC_Order;

defined( 'ABSPATH' ) || exit;

class AllLabelsProcessor {

	/**
	 * @var OcaSdk
	 */
	private $oca_sdk;

	/**
	 * @var PdfA4LabelsProcessor
	 */
	private $pdf_a4_processor;

	/**
	 * @var Pdf10x15LabelsProcessor
	 */
	private $pdf_10x15_processor;

	/**
	 * @var ZplLabelsProcessor
	 */
	private $zpl_processor;

	public function __construct() {
		$this->oca_sdk             = new OcaSdk();
		$this->pdf_a4_processor    = new PdfA4LabelsProcessor();
		$this->pdf_10x15_processor = new Pdf10x15LabelsProcessor();
		$this->zpl_processor       = new ZplLabelsProcessor();
	}

	public function create_order_labels( WC_Order $order ): void {
		$shipping_method = Helper::get_order_shipping_method( $order );

		$oca_number = $shipping_method->get_meta( 'oca_number' );
		if ( empty( $oca_number ) ) {
			Helper::log_error( sprintf( __( 'Tried to create order labels but the order %d is not ready yet', 'wc-oca' ), $order->get_id() ) );
			return;
		}

		$existing_labels = array(
			'A4'    => $this->pdf_a4_processor->label_exists( $order ),
			'10x15' => $this->pdf_10x15_processor->label_exists( $order ),
			'zpl'   => $this->zpl_processor->label_exists( $order ),
		);

		if ( ! $existing_labels['A4'] && ! $existing_labels['zpl'] && ! $existing_labels['10x15'] ) {
			$labels = $this->get_all_shipping_labels( $oca_number );

			if ( ! empty( $labels['pdf-a4'] ) ) {
				$this->pdf_a4_processor->create_label( $order, $labels['pdf-a4'] );
			}
			if ( ! empty( $labels['pdf-10x15'] ) ) {
				$this->pdf_10x15_processor->create_label( $order, $labels['pdf-10x15'] );
			}
			if ( ! empty( $labels['zpl'] ) ) {
				$this->zpl_processor->create_label( $order, $labels['zpl'] );
			}
		}

		// In case previous call did not succeed with all labels, regenerate missing ones.
		$this->pdf_a4_processor->create_label( $order );
		$this->pdf_10x15_processor->create_label( $order );
		$this->zpl_processor->create_label( $order );
	}

	public function delete_all_labels( WC_Order $order ): void {
		$this->pdf_a4_processor->delete_label( $order );
		$this->pdf_10x15_processor->delete_label( $order );
		$this->zpl_processor->delete_label( $order );
	}

	protected function get_all_shipping_labels( string $oca_number ): array {
		$this->oca_sdk->set_data( Helper::get_seller_from_settings() );

		$res = $this->oca_sdk->get_all_shipping_labels( $oca_number );
		return array(
			'pdf-a4'    => ! empty( $res['pdf-a4'] ) ? base64_decode( $res['pdf-a4'] ) : '',
			'pdf-10x15' => ! empty( $res['pdf-10x15'] ) ? base64_decode( $res['pdf-10x15'] ) : '',
			'zpl'       => $res['zpl'],
		);
	}
}
