<?php

namespace CRPlugins\Oca\ShippingMethod;

use CRPlugins\Oca\Helper\Helper;
use CRPlugins\Oca\Orders\Quoter;
use CRPlugins\Oca\ValueObjects\Customer;
use CRPlugins\Oca\ValueObjects\Items;
use CRPlugins\Oca\ValueObjects\Seller;
use CRPlugins\Oca\ValueObjects\ShippingCode;
use CRPlugins\Oca\ValueObjects\ShippingCodes;
use DateTime;
use DateTimeZone;
use Exception;
use WC_Cart;
use WC_Shipping_method;

defined( 'ABSPATH' ) || class_exists( '\WC_Shipping_method' ) || exit;

class WC_Oca extends WC_Shipping_method {

	/**
	 * @var Quoter
	 */
	protected $quoter;

	public function __construct( $instance_id = 0 ) {
		parent::__construct( $instance_id );
		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();
		// Setup general properties.
		$this->setup_properties();
	}

	public function init_form_fields(): void {
		$this->instance_form_fields = array(
			'free_shipping'               => array(
				'title' => __( 'Always free shipping', 'wc-oca' ),
				'type'  => 'checkbox',
			),
			'free_shipping_from_amount'   => array(
				'title'       => __( 'Free shipping from', 'wc-oca' ),
				'type'        => 'decimal',
				'description' => __( 'This adds free shipping when the cart amount is greater than the amount specified. (leave blank to disable)', 'wc-oca' ),
				'desc_tip'    => true,
			),
			'free_shipping_in_time_range' => array(
				'title'       => __( 'Free shipping within time range', 'wc-oca' ),
				'type'        => 'text',
				'description' => __( 'Automatically enables free shipping in a time range, use format 00-24 to enable or leave blank to disable. For example: "04-18" will enable free shipping from 04:00am until 06:00pm in Argentinian timezone.', 'wc-oca' ),
				'desc_tip'    => true,
			),
		);
	}

	private function setup_properties(): void {
		$this->id                 = 'oca';
		$this->method_title       = 'OCA';
		$this->method_description = __( 'Allow your customers receive their orders with OCA', 'wc-oca' );
		$this->title              = 'OCA';
		$this->supports           = array(
			'shipping-zones',
			'instance-settings',
			'instance-settings-modal',
		);
	}

	public function calculate_shipping( $package = array() ): void {
		global $oca_cart_sb;

		$seller = Helper::get_seller_from_settings();
		if ( empty( $seller->get_shipping_codes()->all() ) ) {
			return;
		}

		$customer = Helper::get_customer( WC()->customer );
		if ( ! $customer->address()->has_postcode() ) {
			return;
		}

		$cart      = WC()->cart;
		$cart      = apply_filters( 'wc_oca_cart_before_shipping_calculation', $cart, $customer );
		WC()->cart = $cart;
		try {
			$items = Helper::get_items_from_cart( $cart );

			// Force 1 package.
			if ( Helper::is_enabled( 'proccess_always_one_package' ) ) {
				$default_dimensions = Helper::get_option( 'default_package_size' );
				$default_dimensions = explode( 'x', $default_dimensions );
				$items->set_custom_items_quantity( 1, $default_dimensions );
			}

			// Force price 0.
			if ( ! Helper::is_enabled( 'enable_insurance' ) || $items->totals()['price'] > 9999999 ) {
				$items->set_items_price( 0 );
			}
		} catch ( Exception $e ) {
			if ( Helper::is_debug_enabled() ) {
				Helper::log_warning( __FUNCTION__ . ': ' . $e->getMessage() );
			}
			return;
		}
		$this->quoter = new Quoter();
		$items        = apply_filters( 'wc_oca_items_before_shipping_calculation', $items, $cart, $customer );

		$door_shipping_codes = $seller->get_shipping_codes()->to_door();
		if ( ! empty( $door_shipping_codes ) ) {
			$this->add_shipping_to_door( $door_shipping_codes, $seller, $items, $customer );
		}

		$branch_shipping_codes = $seller->get_shipping_codes()->to_branch();
		if ( ! empty( $branch_shipping_codes ) ) {
			$this->add_shipping_to_branch( $branch_shipping_codes, $seller, $items, $customer );
		} else {
			$oca_cart_sb = array();
		}
	}

	protected function add_shipping_to_door( ShippingCodes $shipping_codes_to_door, Seller $seller, Items $items, Customer $customer ): void {
		$codes_with_price    = $shipping_codes_to_door->with_price();
		$codes_without_price = $shipping_codes_to_door->without_price();

		if ( ! empty( $codes_without_price->all() ) ) {
			$prices = $this->quoter->quote_cart(
				$items,
				$seller,
				$customer,
				$codes_without_price
			);

			if ( ! empty( $prices ) && ! isset( $prices['error'] ) ) {
				$prices = apply_filters( 'wc_oca_door_shipping_rates', $prices, $items );
				foreach ( $prices as $shipping_code => $price ) {
					if ( empty( $price ) ) {
						continue;
					}

					$code  = $codes_without_price->get_by_code( $shipping_code );
					$price = $this->final_price( $price['Total'], WC()->cart );
					$rate  = array(
						'id'        => $this->get_rate_id() . '|door|' . $code->get_code(),
						'label'     => $this->shipping_label( $code, $price ),
						'cost'      => $price,
						'meta_data' => array(
							'shipping_code' => $code->to_array(),
							'oca_packages'  => $items->totals()['quantity'],
						),
					);
					$rate  = apply_filters( 'wc_oca_door_shipping_rate', $rate );
					$this->add_rate( $rate );
				}
			}
		}

		if ( ! empty( $codes_with_price->all() ) ) {
			foreach ( $codes_with_price->all() as $code ) {
				$price = $this->final_price( $code->get_price(), WC()->cart );
				$rate  = array(
					'id'        => $this->get_rate_id() . '|door|' . $code->get_code(),
					'label'     => $this->shipping_label( $code, $price ),
					'cost'      => $price,
					'meta_data' => array(
						'shipping_code' => $code->to_array(),
						'oca_packages'  => $items->totals()['quantity'],
					),
				);
				$rate  = apply_filters( 'wc_oca_door_shipping_rate', $rate );
				$this->add_rate( $rate );
			}
		}
	}

	protected function add_shipping_to_branch( ShippingCodes $shipping_codes_to_branch, Seller $seller, Items $items, Customer $customer ): void {
		global $oca_cart_sb;

		$new_branches = $this->quoter->get_shipping_branches_to( $customer->address()->postcode() );
		if ( ! empty( $new_branches ) && empty( $new_branches['error'] ) ) {
			if ( Helper::is_enabled( 'enable_branch_selector' ) ) {
				$selected_branch = WC()->session->get( 'wc_oca_selected_shipping_branch', '{}' );
				$selected_branch = json_decode( $selected_branch, true );
				if ( empty( $selected_branch ) || ! ShippingBranchHelper::is_same_branch_request( $customer->address()->postcode() ) ) {
					$selected_branch = ShippingBranchHelper::select_first_branch( $new_branches );
				}
				$oca_cart_sb = $new_branches;
				WC()->session->set( 'wc_oca_last_shipping_branch_request', $customer->address()->postcode() );
			} else {
				// If we don't fill the global branches variables, then the branches selector won't come up
				$oca_cart_sb     = array();
				$selected_branch = ShippingBranchHelper::select_first_branch( $new_branches );
			}

			$codes_with_price    = $shipping_codes_to_branch->with_price();
			$codes_without_price = $shipping_codes_to_branch->without_price();

			if ( ! empty( $codes_without_price->all() ) ) {
				$prices = $this->quoter->quote_cart(
					$items,
					$seller,
					$selected_branch,
					$shipping_codes_to_branch
				);
				if ( ! empty( $prices ) && ! isset( $prices['error'] ) ) {
					$prices = apply_filters( 'wc_oca_branch_shipping_rates', $prices, $items, $selected_branch );
					foreach ( $prices as $shipping_code => $price ) {
						if ( empty( $price ) ) {
							continue;
						}

						$code  = $codes_without_price->get_by_code( $shipping_code );
						$price = $this->final_price( $price['Total'], WC()->cart );
						$rate  = array(
							'id'        => $this->get_rate_id() . '|branch|' . $code->get_code(),
							'label'     => $this->shipping_label( $code, $price, $selected_branch ),
							'cost'      => $price,
							'meta_data' => array(
								'shipping_code'      => $code->to_array(),
								'destination_branch' => $selected_branch,
								'oca_packages'       => $items->totals()['quantity'],
							),
						);
						$rate  = apply_filters( 'wc_oca_branch_shipping_rate', $rate );
						$this->add_rate( $rate );
					}
				}
			}

			if ( ! empty( $codes_with_price->all() ) ) {
				foreach ( $codes_with_price->all() as $code ) {
					$price = $this->final_price( $code->get_price(), WC()->cart );
					$rate  = array(
						'id'        => $this->get_rate_id() . '|branch|' . $code->get_code(),
						'label'     => $this->shipping_label( $code, $price, $selected_branch ),
						'cost'      => $price,
						'meta_data' => array(
							'shipping_code'      => $code->to_array(),
							'destination_branch' => $selected_branch,
							'oca_packages'       => $items->totals()['quantity'],
						),
					);
					$rate  = apply_filters( 'wc_oca_door_shipping_rate', $rate );
					$this->add_rate( $rate );
				}
			}
		}
	}

	protected function final_price( $price, WC_Cart $cart ): float {
		if ( $this->get_instance_option( 'free_shipping' ) === 'yes' ) {
			return 0;
		}

		$free_shipping_from_amount = $this->get_instance_option( 'free_shipping_from_amount' );
		if ( ! empty( $free_shipping_from_amount ) && $cart->get_cart_contents_total() > (float) $free_shipping_from_amount ) {
			return 0;
		}

		$free_shipping_in_time_range = $this->get_instance_option( 'free_shipping_in_time_range' );
		if ( ! empty( $free_shipping_in_time_range ) && preg_match( '/(\d{2})-(\d{2})/', $free_shipping_in_time_range, $res ) ) {
			$res[1] = (int) $res[1];
			$res[2] = (int) $res[2];

			$now        = new DateTime( 'now', new DateTimeZone( 'America/Argentina/Buenos_Aires' ) );
			$start_hour = new DateTime( 'now', new DateTimeZone( 'America/Argentina/Buenos_Aires' ) );
			$end_hour   = new DateTime( 'now', new DateTimeZone( 'America/Argentina/Buenos_Aires' ) );

			$start_hour->setTime( $res[1], 0 );
			$end_hour->setTime( $res[2], 0 );
			if ( $res[1] > $res[2] ) {
				// e.g: 23-02
				$end_hour->modify( '+1 day' );
			}

			if ( $now >= $start_hour && $now <= $end_hour ) {
				return 0;
			}
		}

		$coupons = WC()->cart->get_coupons();
		if ( Helper::is_enabled( 'free_shipping_with_coupon' ) && $coupons ) {
			foreach ( $coupons as $code => $coupon ) {
				if ( $coupon->is_valid() && $coupon->get_free_shipping() ) {
					return 0;
				}
			}
		}

		$price            = (float) $price;
		$price_multiplier = Helper::get_option( 'price_multiplier', 1 );
		$price           *= floatval( $price_multiplier );

		if ( Helper::is_enabled( 'round_costs' ) ) {
			$price = ceil( $price ); // ceil to prevent money loss
		}

		return $price;
	}

	protected function shipping_label( ShippingCode $shipping_code, float $price, array $selected_branch = null ): string {
		$shipping_label = $shipping_code->get_name();

		$extra_name_free = Helper::get_option( 'free_shipping_extra_name', null );
		if ( 0.0 === $price && ! empty( $extra_name_free ) ) {
			$shipping_label .= ' ' . $extra_name_free;
		}

		if ( $selected_branch ) {
			$shipping_label .= ' ' . $selected_branch['name'];
		}

		return $shipping_label;
	}
}
