<?php

use CRPlugins\Oca\Helper\Helper;
use CRPlugins\Oca\Orders\AjaxCalls;
use CRPlugins\Oca\Orders\Metabox;
use CRPlugins\Oca\Orders\OrdersList;
use CRPlugins\Oca\ShippingLabels\ShippingLabelsCleanerCron;
use CRPlugins\Oca\ShippingMethod\AjaxCalls as ShippingMethodAjaxCalls;
use CRPlugins\Oca\ShippingMethod\OrderProcessor;
use CRPlugins\Oca\ShippingMethod\ShippingBranchHelper;
use CRPlugins\Oca\ShippingMethod\WC_Oca as ShippingMethodWC_Oca;
use CRPlugins\Oca\Shortcodes\ShippingBranchesShortcode;
use CRPlugins\Oca\Shortcodes\TrackingShortcode;

/**
 * Plugin Name: OCA para WooCommerce
 * Description: Integración entre OCA y WooCommerce
 * Version: 2.6.0
 * Requires PHP: 7.1
 * Author: CRPlugins
 * Author URI: https://crplugins.com.ar
 * Text Domain: wc-oca
 * Domain Path: /i18n/languages/
 * WC requires at least: 4
 * WC tested up to: 7.4.0
 */

defined( 'ABSPATH' ) || exit;

class WC_OCA {

	public const PLUGIN_NAME = 'OCA para WooCommerce';
	public const MAIN_FILE   = __FILE__;
	public const MAIN_DIR    = __DIR__;
	public const PLUGIN_VER  = '2.6.0';

	public function __construct() {
		add_action( 'plugins_loaded', array( $this, 'init' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'admin_register_scripts' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'frontend_register_scripts' ) );
		add_filter( 'plugin_action_links_' . plugin_basename( self::MAIN_FILE ), array( $this, 'create_settings_link' ) );
		add_action( 'before_woocommerce_init', array( $this, 'declare_wc_compatibility' ) );
	}

	public function init(): void {
		if ( ! $this->check_system() ) {
			return;
		}

		// TODO: Replace for autoloader
		require_once __DIR__ . '/ValueObjects/Customer.php';
		require_once __DIR__ . '/ValueObjects/CustomerAddress.php';
		require_once __DIR__ . '/ValueObjects/Item.php';
		require_once __DIR__ . '/ValueObjects/Items.php';
		require_once __DIR__ . '/ValueObjects/SellerAddress.php';
		require_once __DIR__ . '/ValueObjects/ShippingCode.php';
		require_once __DIR__ . '/ValueObjects/ShippingCodes.php';
		require_once __DIR__ . '/ValueObjects/Seller.php';
		require_once __DIR__ . '/Api/ApiConnector.php';
		require_once __DIR__ . '/Api/RestApiInterface.php';
		require_once __DIR__ . '/Api/JsonApi.php';
		require_once __DIR__ . '/Api/OcaApi.php';
		require_once __DIR__ . '/Sdk/OcaSdk.php';
		require_once __DIR__ . '/Helper/LoggerTrait.php';
		require_once __DIR__ . '/Helper/StringsTrait.php';
		require_once __DIR__ . '/Helper/AssetsTrait.php';
		require_once __DIR__ . '/Helper/FilesTrait.php';
		require_once __DIR__ . '/Helper/NoticesTrait.php';
		require_once __DIR__ . '/Helper/SettingsTrait.php';
		require_once __DIR__ . '/Helper/WooCommerceTrait.php';
		require_once __DIR__ . '/Helper/Helper.php';
		require_once __DIR__ . '/Settings/Main.php';
		require_once __DIR__ . '/Settings/AjaxCalls.php';
		require_once __DIR__ . '/Settings/FieldsVerifier.php';
		require_once __DIR__ . '/Settings/FieldFactory.php';
		require_once __DIR__ . '/Settings/Fields/FieldInterface.php';
		require_once __DIR__ . '/Settings/Fields/Field.php';
		require_once __DIR__ . '/Settings/Fields/TextField.php';
		require_once __DIR__ . '/Settings/Fields/DescriptionField.php';
		require_once __DIR__ . '/Settings/Fields/CheckboxField.php';
		require_once __DIR__ . '/Settings/Fields/SelectField.php';
		require_once __DIR__ . '/Settings/Fields/NumberField.php';
		require_once __DIR__ . '/Settings/Fields/PasswordField.php';
		require_once __DIR__ . '/Settings/Fields/OcaShippingCodesField.php';
		require_once __DIR__ . '/Settings/Fields/AjaxButtonField.php';
		require_once __DIR__ . '/Settings/Sections/SectionInterface.php';
		require_once __DIR__ . '/Settings/Sections/Section.php';
		require_once __DIR__ . '/Settings/Sections/SellerSection.php';
		require_once __DIR__ . '/Settings/Sections/OcaSection.php';
		require_once __DIR__ . '/Settings/Sections/PluginSection.php';
		require_once __DIR__ . '/Settings/Sections/IntegrationSection.php';
		require_once __DIR__ . '/Orders/Metabox.php';
		require_once __DIR__ . '/Orders/AjaxCalls.php';
		require_once __DIR__ . '/Orders/OrdersList.php';
		require_once __DIR__ . '/Orders/Quoter.php';
		require_once __DIR__ . '/Shortcodes/TrackingShortcode.php';
		require_once __DIR__ . '/Shortcodes/ShippingBranchesShortcode.php';
		require_once __DIR__ . '/ShippingMethod/WC_Oca.php';
		require_once __DIR__ . '/ShippingMethod/ShippingBranchHelper.php';
		require_once __DIR__ . '/ShippingMethod/AjaxCalls.php';
		require_once __DIR__ . '/ShippingMethod/OrderProcessor.php';
		require_once __DIR__ . '/ShippingLabels/AbstractLabelsProcessor.php';
		require_once __DIR__ . '/ShippingLabels/AllLabelsProcessor.php';
		require_once __DIR__ . '/ShippingLabels/Pdf10x15LabelsProcessor.php';
		require_once __DIR__ . '/ShippingLabels/PdfA4LabelsProcessor.php';
		require_once __DIR__ . '/ShippingLabels/ZplLabelsProcessor.php';
		require_once __DIR__ . '/ShippingLabels/ShippingLabelsCleanerCron.php';
		require_once __DIR__ . '/Hooks.php';
		Helper::init();
		$this->init_classes();
		$this->load_textdomain();
	}

	public function check_system( bool $show_notice = true ): bool {
		require_once ABSPATH . 'wp-admin/includes/plugin.php';
		$system = self::check_components();

		if ( $system['flag'] ) {
			deactivate_plugins( plugin_basename( __FILE__ ) );
			if ( $show_notice ) {
				echo '<div class="notice notice-error is-dismissible">';
                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo '<p>' . sprintf( __( '<strong>%1$s</strong> Requires at least %2$s version %3$s or greater.', 'wc-oca' ), esc_html( self::PLUGIN_NAME ), $system['flag'], $system['version'] ) . '</p>';
				echo '</div>';
			}
			return false;
		}

		if ( ! class_exists( 'WooCommerce' ) ) {
			deactivate_plugins( plugin_basename( __FILE__ ) );
			if ( $show_notice ) {
				echo '<div class="notice notice-error is-dismissible">';
                // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				echo '<p>' . sprintf( __( 'WooCommerce must be active before using <strong>%s</strong>', 'wc-oca' ), esc_html( self::PLUGIN_NAME ) ) . '</p>';
				echo '</div>';
			}
			return false;
		}

		return true;
	}

	private static function check_components(): array {

		global $wp_version;
		$flag    = false;
		$version = false;

		if ( version_compare( PHP_VERSION, '7.1', '<' ) ) {
			$flag    = 'PHP';
			$version = '7.1';
		} elseif ( version_compare( $wp_version, '5.0', '<' ) ) {
			$flag    = 'WordPress';
			$version = '5.0';
		} elseif ( ! defined( 'WC_VERSION' ) || version_compare( WC_VERSION, '4.0', '<' ) ) {
			$flag    = 'WooCommerce';
			$version = '4.0';
		}

		return array(
			'flag'    => $flag,
			'version' => $version,
		);
	}

	public function init_classes(): void {
		new ShippingBranchesShortcode();
		new TrackingShortcode();
		new ShippingBranchHelper();
		new OrderProcessor();
		new OrdersList();
		new Metabox();
		new AjaxCalls();
		new ShippingLabelsCleanerCron();
		new ShippingMethodAjaxCalls();

		add_filter( 'woocommerce_shipping_methods', array( $this, 'add_shipping_method' ) );
	}

	public function load_textdomain() {
		load_plugin_textdomain( 'wc-oca', false, basename( dirname( __FILE__ ) ) . '/i18n/languages' );
	}

	public function admin_register_scripts(): void {
		if ( ! $this->check_system( false ) ) {
			return;
		}
		wp_register_script( 'wc-oca-helper-js', Helper::get_assets_folder_url() . '/js/helper.js', array(), self::PLUGIN_VER, true );
		wp_localize_script(
			'wc-oca-helper-js',
			'wc_oca_helper_settings',
			array(
				'general_css' => Helper::get_general_css(),
				'ajax_url'    => admin_url( 'admin-ajax.php' ),
				'ajax_nonce'  => wp_create_nonce( 'wc-oca' ),
			)
		);
		wp_localize_script(
			'wc-oca-helper-js',
			'wc_oca_translation_texts',
			array(
				'generic_error_try_again' => __( 'There was an error, please try again', 'wc-oca' ),
				'loading'                 => __( 'Loading...', 'wc-oca' ),
			)
		);
		wp_register_script( 'wc-oca-settings-js', Helper::get_assets_folder_url() . '/js/settings.js', array( 'jquery', 'wc-oca-helper-js' ), self::PLUGIN_VER, true );
		wp_register_style( 'wc-oca-settings-css', Helper::get_assets_folder_url() . '/css/settings.css', array(), self::PLUGIN_VER );
		wp_register_style( 'wc-oca-general-css', Helper::get_assets_folder_url() . '/css/general.css', array(), self::PLUGIN_VER );
		wp_register_script( 'wc-oca-orders-list-js', Helper::get_assets_folder_url() . '/js/orders-list.js', array( 'jquery', 'wc-oca-helper-js' ), self::PLUGIN_VER, true );
		wp_register_script( 'wc-oca-orders-js', Helper::get_assets_folder_url() . '/js/orders.js', array( 'jquery', 'wc-oca-helper-js' ), self::PLUGIN_VER, true );
		wp_register_style( 'wc-oca-orders-css', Helper::get_assets_folder_url() . '/css/orders.css', array(), self::PLUGIN_VER );
		wp_register_script( 'wc-oca-reload-js', Helper::get_assets_folder_url() . '/js/reload.js', array(), self::PLUGIN_VER, true );
	}

	public function declare_wc_compatibility() {
		if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
			\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', self::MAIN_FILE, true );
		}
	}

	public function frontend_register_scripts(): void {
		if ( ! $this->check_system( false ) ) {
			return;
		}
		wp_register_script( 'wc-oca-cho-branches-dropdown-js', Helper::get_assets_folder_url() . '/js/cho-branches-dropdown.js', array(), self::PLUGIN_VER, true );
	}

	public function create_settings_link( array $links ): array {
		$link = '<a href="' . esc_url( get_admin_url( null, 'admin.php?page=wc-oca-admin' ) ) . '">' . __( 'Settings', 'wc-oca' ) . '</a>';
		array_unshift( $links, $link );
		return $links;
	}

	public function add_shipping_method( $shipping_methods ): array {
		$shipping_methods['oca'] = ShippingMethodWC_Oca::class;
		return $shipping_methods;
	}
}

new WC_OCA();
